/** \file main.c
 *
 * \brief UART: remote echo.
 *
 * This example program receives characters for the console and sends them back.
 *
 * The relevant registers are:
 * - UDRn: USARTn I/O Data Register
 * - UBRRn: USARTn Baud Rate Register
 * - UCSRnA: USARTn Control and Status Register A
 *   - RXCn: Receive Complete
 *   - DREn: Data Register Empty
 * - UCSRnB: USARTn Control and Status Register B
 *   - RXENn: Receiver Enable
 *   - TXENn: Transmitter Enable
 * - UCSRnC: USARTn Control and Status Register C
 *   - UPMn0: Parity Mode (0 -> None)
 *   - USBSn: Stop Bit Select (0 -> 1 stop bit)
 *   - UCSZn0: Character Size (3 -> 8 data bits)
 *
 * Please see modules 4.2.5 and 4.3.5 for more information.
 *
 * \note
 * A console program sends the characters typed on the keyboard to another device and displays the
 * characters received from that device on the screen. We do not see anything when typing at the
 * console as the characters are directly sent to the remote device. Only characters which we
 * receive are displayed. So the remote device has to send the characters back in order to see
 * them. This is called remote echo.
 *
 * \par Baud Rates (F_CPU = 11059200)
 *      | Baud   | U2X = 0 | U2X = 1 |
 *      |-------:|--------:|--------:|
 *      |   2400 |     287 |     575 |
 *      |   4800 |     143 |     287 |
 *      |   9600 |      71 |     143 |
 *      |  14400 |      47 |      95 |
 *      |  19200 |      35 |      71 |
 *      |  28800 |      23 |      47 |
 *      |  38400 |      17 |      35 |
 *      |  57600 |      11 |      23 |
 *      |  76800 |       8 |      17 |
 *      | 115200 |       5 |      11 |
 *
 * \author    Wolfgang Neff
 * \version   1.1
 * \date      2018-07-22
 *
 * \par History
 *      Created: 2017-11-27 \n
 *      Modified: 2018-07-22
 */

#include <avr/io.h>
#include "board.h"

int main(void)
{
	// We need a buffer to store the data received.
	volatile uint8_t buffer;

	/***** Configure the USART pins *****/
	// The TX pin is output and idle at first. Idle means that it must be set high.
	USB_USART_PORT |= (1 << USB_USART_TX);
	USB_USART_DDR |= (1 << USB_USART_TX);
	// The RX pin is input. We ensure this by clearing DIR although this is default.
	USB_USART_DDR &= ~(1 << USB_USART_RX);

    /***** Configure the USART module *****/
    // We are using the USB-to-USART gateway. Its configuration is 57600 8N1.
    // Baud rate is 57600 bps. See baud rate table.
	USB_USART_UBRR = 11;
	// Line configuration is 8 data bits, no parity, 1 stop bit.
	USB_USART_UCSRC = (0 << UPM10 | 0 << USBS1 | 3 << UCSZ10);
	// Enable receiver and transmitter.
	USB_USART_UCSRB = ((1 << RXEN1) | (1 << TXEN1));

    // We use non-blocking receive and blocking send for the echo.
	while(1)
	{
		// Receive complete? If true the receive complete flag is set.
		if (USB_USART_UCSRA & (1 << RXC1)) {
            // Read data and store it in the buffer.
			buffer = USB_USART_UDR;
			// While the data register empty flag is not set do nothing.
			while(!(USB_USART_UCSRA & (1 << UDRE1)));
			// Now the data register is empty and we can send the data safely.
			USB_USART_UDR = buffer;
		}
	}
}
