/** \file board.h
 *
 * \brief Header file for the MEGA-1284P Xplained evaluation board.
 *
 * This header provides constants for the following components on the board:
 *
 * * \ref header
 * * \ref leds
 * * \ref buttons
 * * \ref gateway
 * * \ref light
 * * \ref temperature
 * * \ref speaker
 * * \ref oscillators
 *
 * \author    Wolfgang Neff
 * \version   1.0
 * \date      2018-07-23
 *
 * \par History
 *      Created: 2017-11-09 \n
 *      Modified: 2018-07-23
 *
 * \note
 *      **USB:** Parameter for the USART-to-USB gateway: 57600 8N1. \n
 *      **F_CPU:** Default 11059200. Use default or set it via Project/Properties/Toolchain/Compiler/Symbols. \n
 *      **FreeROTS:** Timer TC0 used as tick generator. Default tick rate is 1000 ticks per second.
 */

#ifndef MEGA_1284P_XPLAINED_H_
#define MEGA_1284P_XPLAINED_H_

/** @defgroup header Header configuration
 <summary>Pin assignment of the I/O header.</summary>
@{
 \par Header Configuration
 | Pin | Communication Port| Analog Port      | GPIO Port         | Communication Port |
 |-----|-------------------| -----------------|-------------------|--------------------|
 |     |  <B>Jumper J1</B> | <B>Jumper J2</B> | <B>Jumper: J3</B> | <B>Jumper: J4</B>  |
 |  1  |  PC1 (SDA)        | PA0 (ADC0/AREF)  | PB0 (GPIO0)       | PC1 (SDA)          |
 |  2  |  PC0 (SCL)        | PA1 (ADC1)       | PB1 (GPIO1)       | PC0 (SCL)          |
 |  3  |  PD0 (RXD0)       | PA2 (ADC2)       | PB2 (GPIO2)       | PD2 (RXD1)         |
 |  4  |  PD1 (TXD0)       | PA3 (ADC3)       | PB3 (GPIO3)       | PD3 (TXD1)         |
 |  5  |  PB4 (SS0)        | PA4 (ADC4)       | PD4 (GPIO4)       | PD4 (SS1)          |
 |  6  |  PB5 (MOSI)       | PA5 (ADC5)       | PD5 (GPIO5)       | PB5 (MOSI)         |
 |  7  |  PB6 (MISO)       | PA6 (ADC6)       | PC4 (GPIO6)       | PB6 (MISO)         |
 |  8  |  PB7 (SCK)        | PA7 (ADC7)       | PC5 (GPIO7)       | PB7 (SCK)          |
 |  9  |  GND              | GND              | GND               | GND                |
 | 10  |  VCC_P3V3         | VCC_ANA_P3V3     | VCC_USB_VBUS_P5V0 | VCC_P3V3           |

 __Jumpers:__ J1 and J4 share many communication pins. J3 and J4 share pin 5.

 \par Shared Functionality
 | Pin | Jumper J2     | Jumper: J3   |
 |-----|---------------|--------------|
 |  1  |               | SW0, LED0    |
 |  2  |               | SW1, LED2    |
 |  3  |               | SW2, LED3    |
 |  4  |               | LED1         |
 |  5  |               |              |
 |  6  | Filter output | Filter input |
 |  7  | Light sensor  | JTAG         |
 |  8  | NTC sensor    | JTAG         |
@} */

/** @defgroup leds LEDs
 <summary>Constants for the active low LEDs.</summary>
 <remarks>
 There are four active low LEDs on the board. LED0, LED2
 and LED3 are connected with buttons. They will light up
 when the corresponding button is pressed. 
 \verbatim
  VCC_P3V3
     +
     |
     V -> EL17-21UYC/A2
     -
     |
     |
    .-.
    | | 330R
    | |
    '-'
     |
     o---- Button Control (LED0, LED2, LED3)
     |
    .-.
    | | 330R
    | |
    '-'
     |
     |
 PB[3..0]
 \endverbatim
 </remarks>
@{ */
/****** Active low LEDs ******/
#define LED_DDR DDRB
#define LED_PORT PORTB
#define LED_MASK 0x0F

#define LED0 PINB0
#define LED1 PINB3
#define LED2 PINB1
#define LED3 PINB2
/// @}

/** @defgroup buttons Buttons
 <summary>Constants for the active low push buttons.</summary>
 <remarks>
 There are three active low push buttons on the board. They are
 connected LEDs. The LEDs will light up or shine brighter when
 the corresponding button is pressed. The button pins control
 also the LEDs. Therefore the pins have to be time multiplexed
 if both are used.
 \verbatim
  PB[2..0]
     |
     |
    .-.
    | | 330R
    | |
    '-'
     |
     |
   | o
 |=|>
   | o
     |
     |
    ===
    GND
 \endverbatim
 </remarks>
@{ */ 
/****** Active low push buttons ******/
#define BUTTON_DDR DDRB
#define BUTTON_PIN PINB
#define BUTTON_PORT PORTB
#define BUTTON_MASK 0x07

#define BUTTON0 PINB0
#define BUTTON1 PINB1
#define BUTTON2 PINB2

/// \def BUTTON_PRESSED(BUTTON)
/// <summary>Check if push button is pressed</summary>
/// <param name="BUTTON">Number of button to be checked.</param>
#define BUTTON_PRESSED(BUTTON) (!(BUTTON_PIN & (1 << BUTTON)))
/// @}

/** @defgroup gateway Gateway
 <summary>Constants for the USART-to-USB gateway.</summary>
 <remarks>
 USART1 is connected with a USART-to-USB gateway.
 The configuration for the gateway is: 57600 8N1.
 </remarks>
@{ */
/****** USART-to-USB gateway ******/
#define USB_USART_RX PIND2
#define USB_USART_TX PIND3
#define USB_USART_DDR DDRD
#define USB_USART_PIN PIND
#define USB_USART_PORT PORTD

#define USB_USART_UDR UDR1
#define USB_USART_UBRR UBRR1
#define USB_USART_UCSRA UCSR1A
#define USB_USART_UCSRB UCSR1B
#define USB_USART_UCSRC UCSR1C

#define USB_USART_RXC_vect USART1_RX_vect
#define USB_USART_TXC_vect USART1_TX_vect
#define USB_USART_DRE_vect USART1_UDRE_vect

#define USB_USART_SPEED 57600
#define USB_USART_BAUDRATE 11
#define USB_USART_CONFIG (3 << UCSZ10)
/// @}

/****** Sensors ******/
#define SENSOR_DDR PORTA

/** @defgroup light Light sensor
 <summary>Constants for the light sensor.</summary>
 <remarks>
 There is an ambient light sensor on the board.
 Darkness provides higher voltages. Please consult
 the schematic for details. 
 \verbatim
    VCC_P3V3
       +
       |
      .-.
      | | 100k
      | |
      '-'
       |
       o--- PA6/ADC6
       |
 ->  |/
    -|   Phototransistor
 ->  |>  TEMT6000
       |
       |
      ===
      GND
 \endverbatim
 </remarks>
@{ */ 
/****** Light sensor ******/
#define LIGHT_SENSOR PINA6
/// @}

/** @defgroup temperature Temperature sensor
 <summary>Constants for the temperature sensor.</summary>
 <remarks>
 There is a voltage divider with an NTC on the board. It is
 used as temperature sensor. It has an active low enable input
 and lower voltages for higher temperatures. Please consult
 the schematic for details. 
 \verbatim
 VCC_P3V3
    +
    |
   .-.
   | | 220k
   | |
   '-'
    |
    o---- PA7/ADC7
    |
   .-.
   | |<-- NCP18WF104J03RB
   | |
   '-'
    |
   ===
   GND
 \endverbatim
 </remarks>
@{ */ 
/****** Temperature sensor ******/
#define TEMPERATURE_SENSOR PINA7
/// @}

/** @defgroup filter RC lowpass filter
 <summary>Constants for the RC lowpass filter.</summary>
 <remarks>
 There is a speaker on the board. It has an active low
 shutdown input. Please consult the schematic for details.
 \verbatim
             ___
  PD5/IN ---|___|--o--- PA5/ADC5/OUT
             1k    |
                   |
                  --- 1u
                  ---
                   |
                   |
                  ===
                  GND
 \endverbatim
 </remarks>
@{ */ 
/****** Speaker ******/
#define FILTER_INPUT PIND5
#define FILTER_INPUT_DDR DDRD
#define FILTER_INPUT_PORT PORTD
#define FILTER_OUTPUT PINA5
/// @}

/** @defgroup oscillators Oscillators
 <summary>Constants for the oscillators.</summary>
@{ */
/****** Internal oscillator ******/
#define OSC_INTERNAL_HZ 1000000L
#define OSC_EXTERNAL_HZ 11059200L

/****** Default oscillator ******/
#define OSC_DEFAULT_HZ OSC_EXTERNAL_HZ

/****** F_CPU ******/
#ifndef F_CPU
#define F_CPU OSC_DEFAULT_HZ
#warning "F_CPU initialized with default value"
#endif
/// @}

#endif /* MEGA_1284P_XPLAINED_H_ */
