/** \file main.c
 *
 * \brief PWM: dim an LED.
 *
 * This example program dims LED1 with the aid of pulse-width modulation.
 *
 * The relevant registers are:
 * - OCR0A: Output Compare Register A
 * - TCCR0A: Timer/Counter 0 Control Register A
 *   - COM0A0: Compare Output Mode (2 -> non-inverting mode, 3-> inverting mode) 
 *   - WGM00: Waveform Generation Mode (3 -> Fast PWM)
 * - TCCR0B: Timer/Counter 0 Control Register B
 *   - CS00: Clock Select (0 -> timer/counter off)
 *
 * The value for OCR0A is calculated by OCR0A = 255*d where
 * - d: duty cycle
 * 
 * Please see modules 4.2.4 and 4.3.4 for more information.
 *
 * \author    Wolfgang Neff
 * \version   1.1
 * \date      2018-07-22
 *
 * \par History
 *      Created: 2017-11-28 \n
 *      Modified: 2018-07-22
 */

#include <avr/io.h>

#include "board.h"

// We want to dim LED1. LED1 is on port B (PB3) and is connected to OC0A.
// Therefore we have to use TC0 with compare register A. The LED is active
// low. This has to be taken into account when handling the dimming. We have
// two options to handle this issue:
// - We can invert the duty cycle: 0% => LED on, 25% => LED dimmed to 75% etc.
// - We can invert the pin: the active low LED becomes active high.
int main(void)
{
	// Initialize the LED.
	LED_DDR = (1 << LED1);
	LED_PORT = (1 << LED1);

	// Initialize the push-buttons.
	BUTTON_PORT |= ((1 << BUTTON0) | (1 << BUTTON1) | (1 << BUTTON2));
	/***** Initialize the PWM *****/
	// Enable compare register A and inverting fast PWM mode.
	TCCR0A = ((3 << COM0A0) | (3 << WGM00));
	// Initial duty cycle: 100%.
	OCR0A = 0xFF;
	// Turn on the clock.
	TCCR0B = (1 << CS00);

	while(1)
	{
		if (!(BUTTON_PIN & (1 << BUTTON0))) {
			OCR0A = 0xFF;     // Duty cycle 100%: LED not dimmed
		}
		if (!(BUTTON_PIN & (1 << BUTTON1))) {
			OCR0A = 0xFF/2;   // Duty cycle 50%: LED dimmed to 50%
		}
		if (!(BUTTON_PIN & (1 << BUTTON2))) {
			OCR0A = 0xFF/4;   // Duty cycle 25%: LED dimmed to 25%
		}
	}
}
