/** \file main.c
 *
 * \brief ADC: read the temperature sensor.
 *
 * This example program reads the temperature sensor and prints the result
 * of the ADC conversion at the console.
 *
 * The relevant registers are:
 * - ADC: ADC Data Register
 * - ADMUX: ADC Multiplexer Selection Register
 *   - REFS0: Reference Selection (3 -> internal 2.56 V)
 *   - MUX0: Analog Channel Selection (n -> ADCn)
 * - ADCSRA: ADC Control and Status Register A
 *   - ADEN: ADC Enable
 *   - ADSC: ADC Start Conversion
 *   - ADPS0: ADC Prescaler Select (6 -> prescaler 64)

 *
 * The ADC conversion result is calculated by ADC = 1024*Vin/Vref.
 * The ADC input clock frequency ranges is from 50 kHz to 200 kHz.
 *
 * Please see modules 4.2.4 and 4.3.4 for more information.
 *
 * \author    Wolfgang Neff
 * \version   1.1
 * \date      2018-07-22
 *
 * \par History
 *      Created: 2017-11-28 \n
 *      Modified: 2018-07-22
 */

#include <avr/io.h>
#include <util/delay.h>

#include "board.h"
#include "console.h"

// We use the ADC to read the temperature sensor. The temperature sensor
// produces lower voltages if the terperature increases. We use reference
// voltage of 2.56V for the measurement. The input clock frequency must
// be between 50kHz and 200kH. So we use a clock divider factor of 64.
int main(void)
{
	volatile uint16_t temperature;

	console_init();

	/***** Initialize the ADC. *****/
	// Select the internal 2.56V voltage reference and the temperature
	// sensor as analog input. 
	ADMUX = (3 << REFS0) | (TEMPERATURE_SENSOR << MUX0);
	// Select prescaler division factor 64 (173 kHz) and enable ADC. 
	ADCSRA = (1 << ADEN) | (6 << ADPS0);

	while(1)
	{
		// Start conversion.
		ADCSRA |= (1<<ADSC);
		// Wait until the conversion has finished.
		while (ADCSRA & (1<<ADSC));
		// Read the result of the conversion.
		temperature = ADC;
		// Print result
		printf("%d\n", temperature);
		// Wait a bit until starting the next conversion.
		_delay_ms(500);
	}
}
