/** \file main.c
 *
 * \brief SPI: data transfer.
 *
 * This example program transmits and receives data via the Serial Peripheral Interface.
 *
 * The relevant registers are:
 * - SPCR: SPI Control Register
 *   - SPE: SPI Enable
 *   - MSTR: Master/Slave Select
 *   - SPR0: Clock Rate Select (3 -> Prescaler: 128)
 * - SPDR: SPI Data Register
 * - SPSR: SPI Status Register
 *   - SPIF: SPI Interrupt Flag
 *
 * Please see module 5.3.3 for more information.
 *
 * \author    Wolfgang Neff
 * \version   1.1
 * \date      2018-07-22
 *
 * \par History
 *      Created: 2017-12-09 \n
 *      Modified: 2018-07-22
 */

#include <avr/io.h>
#include <util/delay.h>

#include "board.h"

#define SPI_DDR DDRB
#define SPI_PORT PORTB

#define SPI_SS PINB4    // Slave Select
#define SPI_MOSI PINB5  // Master Out, Slave In
#define SPI_MISO PINB6  // Master In, Slave Out
#define SPI_SCK  PINB7  // Serial Clock

#define SPI_DATA 'A'

// We want to transmit data via the SPI interface on port B. The microcontroller
// is the master of the transmission. MISO is input. MOSI, SS and SCK are output.
// We use a prescaler of 128. Therefore the transmission speed is 86.4 kbps. The
// slave select line is active low. At the beginning the slave is deselected.
int main(void)
{
	/***** Initialize SPI. *****/
	// Deactivate the slave.
	SPI_PORT = (1 << SPI_SS);
	// Make SS, MOSI and SCK output lines.
	SPI_DDR = ((1 << SPI_SS) | (1 << SPI_MOSI) | (1 << SPI_SCK));
	// Enable SPI, select master mode and set the prescaler factor to 128.
	SPCR = ((1 << SPE) | (1 << MSTR) | (3 << SPR0));

    while(1)
    {
		// Select the slave.
		SPI_PORT &= ~(1 << SPI_SS);
		// Send the data.
		SPDR = SPI_DATA;
		// While transmission has not yet finished do nothing.
		while (!(SPSR & (1 << SPIF)));
		// Deselect the slave.
		SPI_PORT = (1 << SPI_SS);
		// Wait a bit before starting the next transmission.
		_delay_ms(1000);
    }
}
