/** \file main.c
 *
 * \brief I2C: data transfer.
 *
 * This example program sends data via the Inter-Integrated Circuit Interface. The
 * slave has the address 0x20. Pull-up resistors have to be present on the bus.
 *
 * The relevant registers are:
 * - TWCR: TWI Control Register
 *   - TWINT: TWI Interrupt Flag
 *   - TWSTA: TWI START Condition
 *   - TWSTO: TWI STOP Condition
 *   - TWEN: TWI Enable
 * - TWDR: TWI Data Register
 * - TWBR: TWI Bit Rate Register
 *
 * The Baud rate is calculated by TWBR = F_CPU/(8*F_I2C)-2 if TWPS = 0. The
 * maximum transmission rate is standard mode (100 kbit/s, F_I2C=100000).
 *
 * Please see module 5.3.3 for more information.
 *
 * \note
 * Atmel calls the bus Two-Wire Serial Interface (TWI). So the registers
 * and the bit names start with TWI.
 *
 * \par Bus Speeds (F_CPU = 11059200, TWPS = 0)
 *      | Mode      | Speed      | TWBR | Error |
 *      |-----------|------------|------|-------|
 *      | Low-speed |  10 kbit/s |  136 |    0% |
 *      | Standard  | 100 kbit/s |   12 |   -1% |
 *
 * \author    Wolfgang Neff
 * \version   1.1
 * \date      2018-07-22
 *
 * \par History
 *      Created: 2017-12-09 \n
 *      Modified: 2018-07-22
 */

#include <avr/io.h>
#include <util/delay.h>

#include "board.h"

#define I2C_ADDRESS 0x20

// We want to transmit data via the I2C interface on port C. The microcontroller
// is the master of the transmission and sends data to a slave with address 0x20.
// We use the standard transmission speed (100 kbit/s) and ensure that the bus
// state is idle.
int main(void)
{
	/***** Initialize I2C *****/
	// Select standard mode speed.
    TWBR = 12;

	while(1)
	{
	    /***** Open write transmission. *****/
		// Enable the I2C module, send the start condition and clear the
		// TWI interrupt flag to start the transmission.
		TWCR = (1 << TWINT) | (1 << TWSTA) | (1 <<TWEN);
		// Wait until the transmission has finished.
		while (!(TWCR & (1 << TWINT)));
		// Send slave address with write indicator.
		TWDR = I2C_ADDRESS<<1;
		TWCR = (1 << TWINT) | (1 <<TWEN);
		while (!(TWCR & (1 << TWINT)));

	    /***** Send the data. *****/
		// Put the data onto the bus.
		TWDR = 'A';
		// Start the transmission.
		TWCR = (1 << TWINT) | (1 <<TWEN);
		// Wait until the transmission has finished.
		while (!(TWCR & (1 << TWINT)));

	    /***** Close transmission *****/
		// Send stop condition.
		TWCR = (1 << TWINT) | (1 << TWSTO) | (1 <<TWEN);

		// Wait a bit before starting the next transmission.
		_delay_ms(1000);
	}
}
